import time

from dflow import InputParameter, Inputs, Step, Steps, Workflow
from dflow.python import OP, OPIO, OPIOSign, PythonOPTemplate, upload_packages

if "__file__" in locals():
    upload_packages.append(__file__)


class Plus1(OP):
    def __init__(self):
        pass

    @classmethod
    def get_input_sign(cls):
        return OPIOSign({
            'iter': int
        })

    @classmethod
    def get_output_sign(cls):
        return OPIOSign({
            'iter': int
        })

    @OP.exec_sign_check
    def execute(
            self,
            op_in: OPIO,
    ) -> OPIO:
        print("This is iter %i" % op_in["iter"])
        return OPIO({
            'iter': op_in['iter'] + 1
        })


def test_recurse():
    steps = Steps(name="iter", inputs=Inputs(
        parameters={"iter": InputParameter(value=0),
                    "limit": InputParameter(value=5)}))
    plus1 = Step(name="plus1",
                 template=PythonOPTemplate(Plus1,
                                           image="python:3.8"),
                 parameters={"iter": steps.inputs.parameters["iter"]},
                 key="iter-%s" % steps.inputs.parameters["iter"])
    steps.add(plus1)
    next = Step(name="next", template=steps,
                parameters={"iter": plus1.outputs.parameters["iter"]},
                when="%s < %s" % (
                    plus1.outputs.parameters["iter"],
                    steps.inputs.parameters["limit"]))
    steps.add(next)

    wf = Workflow("recurse", steps=steps)
    wf.submit()

    while wf.query_status() in ["Pending", "Running"]:
        time.sleep(1)

    assert(wf.query_status() == "Succeeded")
    step = wf.query_step(key="iter-4")[0]
    assert(step.phase == "Succeeded")
    assert step.outputs.parameters["iter"].value == 5


if __name__ == "__main__":
    test_recurse()



import json
import tarfile
from collections.abc import MutableMapping
from pathlib import Path
from typing import Any, Dict, List, Optional, Set, Union

from ..common import CustomHandler, S3Artifact, jsonpickle
from ..config import config
from ..io import PVC, type_to_str


class NestedDictBase:
    pass


class NestedDictStr(NestedDictBase):
    pass


class NestedDictPath(NestedDictBase):
    pass


class HDF5Dataset:
    def __init__(self, file, key):
        self.file = file
        self.key = key

    @property
    def dataset(self):
        return self.file[self.key]

    def __deepcopy__(self, memo=None):
        return self

    def is_none(self):
        return self.dataset.attrs.get("type") == "null"

    def get_data(self):
        if self.is_none():
            return None
        data = self.dataset[()]
        if self.dataset.attrs.get("dtype") == "utf-8":
            data = data.decode("utf-8")
        elif self.dataset.attrs.get("dtype") == "binary":
            data = data.tobytes()
        return data

    def recover(self):
        if self.dataset.attrs["type"] == "file":
            path = Path(self.dataset.attrs["path"])
            if path.is_absolute():
                path = path.relative_to(path.root)
            path.parent.mkdir(parents=True, exist_ok=True)
            data = self.get_data()
            if isinstance(data, str):
                path.write_text(data)
            elif isinstance(data, bytes):
                path.write_bytes(data)
            return path
        elif self.dataset.attrs["type"] == "dir":
            path = Path(self.dataset.attrs["path"])
            if path.is_absolute():
                path = path.relative_to(path.root)
            path.parent.mkdir(parents=True, exist_ok=True)
            tgz_path = path.parent / (path.name + ".tgz")
            tgz_path.write_bytes(self.get_data())
            tf = tarfile.open(tgz_path, "r:gz")
            tf.extractall(".")
            tf.close()
            return path
        else:
            return self.get_data()


class HDF5Datasets:
    pass


NestedDict = {
    str: NestedDictStr,
    Path: NestedDictPath,
}

ArtifactAllowedTypes = [str, Path, Set[str], Set[Path], List[str], List[Path],
                        Dict[str, str], Dict[str, Path], NestedDict[str],
                        NestedDict[Path]]
for t in ArtifactAllowedTypes.copy():
    ArtifactAllowedTypes.append(Union[t, HDF5Datasets])
ArtifactAllowedTypes.append(HDF5Datasets)


@CustomHandler.handles
class Artifact:
    """
    OPIO signature of artifact

    Args:
        type: str, Path, Set[str], Set[Path], List[str], List[Path],
            Dict[str, str], Dict[str, Path], NestedDict[str] or
            NestedDict[Path]
        archive: compress format of the artifact, None for no compression
        save: place to store the output artifact instead of default storage,
            can be a list
        optional: optional input artifact or not
        global_name: global name of the artifact within the workflow
    """

    def __init__(
            self,
            type: Any,
            archive: str = "default",
            save: List[Union[PVC, S3Artifact]] = None,
            optional: bool = False,
            global_name: Optional[str] = None,
            sub_path: bool = True,
            **kwargs,
    ) -> None:
        self.type = type
        if archive == "default":
            archive = config["archive_mode"]
        self.archive = archive
        self.save = save
        self.optional = optional
        self.global_name = global_name
        self.sub_path = sub_path
        for k, v in kwargs.items():
            self.__setattr__(k, v)

    def __setattr__(self, key, value):
        if key == "type":
            assert (value in ArtifactAllowedTypes), "%s is not allowed" \
                                                    "artifact type, only %s " \
                                                    "are allowed." % (
                                                        value,
                                                        ArtifactAllowedTypes)
        super().__setattr__(key, value)

    def to_dict(self):
        return {
            "type": type_to_str(self.type),
            "archive": self.archive,
            "save": self.save,
            "optional": self.optional,
            "global_name": self.global_name,
            "sub_path": self.sub_path,
        }

    @classmethod
    def from_dict(cls, d):
        if isinstance(d["type"], str):
            d["type"] = {type_to_str(t): t for t in ArtifactAllowedTypes}[
                d["type"]]
        return cls(**d)

    def __repr__(self):
        return "Artifact(type=%s, optional=%s, sub_path=%s)" % (
            type_to_str(self.type), self.optional, self.sub_path)

    def __call__(self, *args, **kwargs):
        self.type(*args, **kwargs)


@CustomHandler.handles
class Parameter:
    """
    OPIO signature of parameter

    Args:
        type: parameter type
        global_name: global name of the parameter within the workflow
        default: default value of the parameter
    """

    def __init__(
            self,
            type: Any,
            global_name: Optional[str] = None,
            **kwargs,
    ) -> None:
        self.type = type
        self.global_name = global_name
        if "default" in kwargs:
            self.default = kwargs["default"]
        for k, v in kwargs.items():
            self.__setattr__(k, v)

    def __repr__(self):
        default = ""
        if hasattr(self, "default"):
            try:
                default = ", default=%s" % json.dumps(self.default)
            except Exception:
                default = ", default=jsonpickle.loads('%s')" % \
                    jsonpickle.dumps(self.default)
        return "Parameter(type=%s%s)" % (type_to_str(self.type), default)

    def to_dict(self):
        d = {
            "type": type_to_str(self.type),
            "global_name": self.global_name,
        }
        if hasattr(self, "default"):
            d["default"] = self.default
        return d

    @classmethod
    def from_dict(cls, d):
        return cls(**d)


@CustomHandler.handles
class BigParameter:
    """
    OPIO signature of big parameter

    Args:
        type: parameter type
    """

    def __init__(
            self,
            type: Any,
            **kwargs,
    ) -> None:
        self.type = type
        if "default" in kwargs:
            self.default = kwargs["default"]
        for k, v in kwargs.items():
            self.__setattr__(k, v)

    def __repr__(self):
        default = ""
        if hasattr(self, "default"):
            try:
                default = ", default=%s" % json.dumps(self.default)
            except Exception:
                default = ", default=jsonpickle.loads('%s')" % \
                    jsonpickle.dumps(self.default)
        return "BigParameter(type=%s%s)" % (type_to_str(self.type), default)

    def to_dict(self):
        d = {
            "type": type_to_str(self.type),
        }
        if hasattr(self, "default"):
            d["default"] = self.default
        return d

    @classmethod
    def from_dict(cls, d):
        return cls(**d)


class OPIOSign(MutableMapping):
    """The signature of OPIO.
    A signature of OPIO includes the key and its typing
    """

    def __init__(
            self,
            *args,
            **kwargs
    ):
        self._data = {}
        self._data = dict(*args, **kwargs)

    def __getitem__(
            self,
            key: str,
    ) -> Any:
        """Get the type hint of the key
        """
        return self._data[key]

    def __setitem__(
            self,
            key: str,
            value: Any,
    ) -> None:
        """Set the type hint of the key
        """
        self._data[key] = value

    def __delitem__(
            self,
            key: str,
    ) -> None:
        del self._data[key]

    def __iter__(self):
        return iter(self._data)

    def __len__(self):
        return len(self._data)

    def __repr__(self):
        return str(self._data)


class OPIO(MutableMapping):
    def __init__(
            self,
            *args,
            **kwargs
    ):
        self._data = {}
        self._data = dict(*args, **kwargs)

    def __getitem__(
            self,
            key: str,
    ) -> Any:
        return self._data[key]

    def __setitem__(
            self,
            key: str,
            value: Any,
    ) -> None:
        self._data[key] = value

    def __delitem__(
            self,
            key: str,
    ) -> None:
        del self._data[key]

    def __iter__(self):
        return iter(self._data)

    def __len__(self):
        return len(self._data)

    def __repr__(self):
        return str(self._data)



# coding: utf-8

"""
    Argo Server API

    You can get examples of requests and responses by using the CLI with `--gloglevel=9`, e.g. `argo list --gloglevel=9`  # noqa: E501

    The version of the OpenAPI document: v2.12.2
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from argo.workflows.client.configuration import Configuration


class V1alpha1LifecycleHook(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'arguments': 'V1alpha1Arguments',
        'expression': 'str',
        'template': 'str',
        'template_ref': 'V1alpha1TemplateRef',
    }

    attribute_map = {
        'arguments': 'arguments',
        'expression': 'expression',
        'template': 'template',
        'template_ref': 'templateRef'
    }

    def __init__(self, arguments=None, expression=None, template=None, template_ref=None, local_vars_configuration=None):  # noqa: E501
        """V1alpha1LifecycleHook - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._arguments = None
        self._expression = None
        self._template = None
        self._template_ref = None

        if arguments is not None:
            self.arguments = arguments
        if expression is not None:
            self.expression = expression
        if template is not None:
            self.template = template
        if template_ref is not None:
            self.template_ref = template_ref

    @property
    def arguments(self):
        """Gets the arguments of this V1alpha1LifecycleHook.  # noqa: E501


        :return: The arguments of this V1alpha1LifecycleHook.  # noqa: E501
        :rtype: V1alpha1Arguments
        """
        return self._arguments

    @arguments.setter
    def arguments(self, arguments):
        """Sets the arguments of this V1alpha1LifecycleHook.


        :param arguments: The arguments of this V1alpha1LifecycleHook.  # noqa: E501
        :type: V1alpha1Arguments
        """

        self._arguments = arguments

    @property
    def expression(self):
        """Gets the expression of this V1alpha1LifecycleHook.  # noqa: E501


        :return: The expression of this V1alpha1LifecycleHook.  # noqa: E501
        :rtype: V1alpha1ContinueOn
        """
        return self._expression

    @expression.setter
    def expression(self, expression):
        """Sets the expression of this V1alpha1LifecycleHook.


        :param expression: The expression of this V1alpha1LifecycleHook.  # noqa: E501
        :type: V1alpha1ContinueOn
        """

        self._expression = expression

    @property
    def template(self):
        """Gets the template of this V1alpha1LifecycleHook.  # noqa: E501

        Template is the name of the template to execute at the hook  # noqa: E501

        :return: The template of this V1alpha1LifecycleHook.  # noqa: E501
        :rtype: str
        """
        return self._template

    @template.setter
    def template(self, template):
        """Sets the template of this V1alpha1LifecycleHook.

        Template is the name of the template to execute at the hook  # noqa: E501

        :param template: The template of this V1alpha1LifecycleHook.  # noqa: E501
        :type: str
        """

        self._template = template

    @property
    def template_ref(self):
        """Gets the template_ref of this V1alpha1LifecycleHook.  # noqa: E501


        :return: The template_ref of this V1alpha1LifecycleHook.  # noqa: E501
        :rtype: V1alpha1TemplateRef
        """
        return self._template_ref

    @template_ref.setter
    def template_ref(self, template_ref):
        """Sets the template_ref of this V1alpha1LifecycleHook.


        :param template_ref: The template_ref of this V1alpha1LifecycleHook.  # noqa: E501
        :type: V1alpha1TemplateRef
        """

        self._template_ref = template_ref

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1alpha1LifecycleHook):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1alpha1LifecycleHook):
            return True

        return self.to_dict() != other.to_dict()



import ast
import sys
from importlib.abc import MetaPathFinder
from importlib.machinery import SourceFileLoader
from importlib.util import cache_from_source, decode_source
from inspect import isclass
from typing import Iterable, Type
from unittest.mock import patch


# The name of this function is magical
def _call_with_frames_removed(f, *args, **kwargs):
    return f(*args, **kwargs)


def optimized_cache_from_source(path, debug_override=None):
    return cache_from_source(path, debug_override, optimization='typeguard')


class TypeguardTransformer(ast.NodeVisitor):
    def __init__(self) -> None:
        self._parents = []

    def visit_Module(self, node: ast.Module):
        # Insert "import typeguard" after any "from __future__ ..." imports
        for i, child in enumerate(node.body):
            if isinstance(child, ast.ImportFrom) and child.module == '__future__':  # noqa: E501
                continue
            elif isinstance(child, ast.Expr) and isinstance(child.value, ast.Str):  # noqa: E501
                continue  # module docstring
            else:
                node.body.insert(i, ast.Import(names=[ast.alias('typeguard', None)]))  # noqa: E501
                break

        self._parents.append(node)
        self.generic_visit(node)
        self._parents.pop()
        return node

    def visit_ClassDef(self, node: ast.ClassDef):
        node.decorator_list.append(
            ast.Attribute(ast.Name(id='typeguard', ctx=ast.Load()), 'typechecked', ast.Load())  # noqa: E501
        )
        self._parents.append(node)
        self.generic_visit(node)
        self._parents.pop()
        return node

    def visit_FunctionDef(self, node: ast.FunctionDef):
        # Let the class level decorator handle the methods of a class
        if isinstance(self._parents[-1], ast.ClassDef):
            return node

        has_annotated_args = any(arg for arg in node.args.args if arg.annotation)  # noqa: E501
        has_annotated_return = bool(node.returns)
        if has_annotated_args or has_annotated_return:
            node.decorator_list.insert(
                0,
                ast.Attribute(ast.Name(id='typeguard', ctx=ast.Load()), 'typechecked', ast.Load())  # noqa: E501
            )

        self._parents.append(node)
        self.generic_visit(node)
        self._parents.pop()
        return node


class TypeguardLoader(SourceFileLoader):
    def source_to_code(self, data, path, *, _optimize=-1):
        source = decode_source(data)
        tree = _call_with_frames_removed(compile, source, path, 'exec', ast.PyCF_ONLY_AST,  # noqa: E501
                                         dont_inherit=True, optimize=_optimize)
        tree = TypeguardTransformer().visit(tree)
        ast.fix_missing_locations(tree)
        return _call_with_frames_removed(compile, tree, path, 'exec',
                                         dont_inherit=True, optimize=_optimize)

    def exec_module(self, module):
        # Use a custom optimization marker – the import lock should make this monkey patch safe  # noqa: E501
        with patch('importlib._bootstrap_external.cache_from_source', optimized_cache_from_source):  # noqa: E501
            return super().exec_module(module)


class TypeguardFinder(MetaPathFinder):
    """
    Wraps another path finder and instruments the module with ``@typechecked`` if
    :meth:`should_instrument` returns ``True``.

    Should not be used directly, but rather via :func:`~.install_import_hook`.

    .. versionadded:: 2.6

    """  # noqa: E501

    def __init__(self, packages, original_pathfinder):
        self.packages = packages
        self._original_pathfinder = original_pathfinder

    def find_spec(self, fullname, path=None, target=None):
        if self.should_instrument(fullname):
            spec = self._original_pathfinder.find_spec(fullname, path, target)
            if spec is not None and isinstance(spec.loader, SourceFileLoader):
                spec.loader = TypeguardLoader(spec.loader.name, spec.loader.path)  # noqa: E501
                return spec

        return None

    def should_instrument(self, module_name: str) -> bool:
        """
        Determine whether the module with the given name should be instrumented.

        :param module_name: full name of the module that is about to be imported (e.g. ``xyz.abc``)

        """  # noqa: E501
        for package in self.packages:
            if module_name == package or module_name.startswith(package + '.'):
                return True

        return False


class ImportHookManager:
    def __init__(self, hook: MetaPathFinder):
        self.hook = hook

    def __enter__(self):
        pass

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.uninstall()

    def uninstall(self):
        try:
            sys.meta_path.remove(self.hook)
        except ValueError:
            pass  # already removed


def install_import_hook(packages: Iterable[str], *,
                        cls: Type[TypeguardFinder] = TypeguardFinder) -> ImportHookManager:  # noqa: E501
    """
    Install an import hook that decorates classes and functions with ``@typechecked``.

    This only affects modules loaded **after** this hook has been installed.

    :return: a context manager that uninstalls the hook on exit (or when you call ``.uninstall()``)

    .. versionadded:: 2.6

    """  # noqa: E501
    if isinstance(packages, str):
        packages = [packages]

    for i, finder in enumerate(sys.meta_path):
        if isclass(finder) and finder.__name__ == 'PathFinder' and hasattr(finder, 'find_spec'):  # noqa: E501
            break
    else:
        raise RuntimeError('Cannot find a PathFinder in sys.meta_path')

    hook = cls(packages, finder)
    sys.meta_path.insert(0, hook)
    return ImportHookManager(hook)



# coding: utf-8

"""
    Argo Server API

    You can get examples of requests and responses by using the CLI with `--gloglevel=9`, e.g. `argo list --gloglevel=9`  # noqa: E501

    The version of the OpenAPI document: v2.12.2
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from argo.workflows.client.configuration import Configuration


class V1alpha1DAGTask(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'arguments': 'V1alpha1Arguments',
        'continue_on': 'V1alpha1ContinueOn',
        'dependencies': 'list[str]',
        'depends': 'str',
        'name': 'str',
        'on_exit': 'str',
        'template': 'str',
        'template_ref': 'V1alpha1TemplateRef',
        'when': 'str',
        'with_items': 'list[object]',
        'with_param': 'str',
        'with_sequence': 'V1alpha1Sequence',
        'hooks': 'dict(str, V1alpha1LifecycleHook)'
    }

    attribute_map = {
        'arguments': 'arguments',
        'continue_on': 'continueOn',
        'dependencies': 'dependencies',
        'depends': 'depends',
        'name': 'name',
        'on_exit': 'onExit',
        'template': 'template',
        'template_ref': 'templateRef',
        'when': 'when',
        'with_items': 'withItems',
        'with_param': 'withParam',
        'with_sequence': 'withSequence',
        'hooks': 'hooks'
    }

    def __init__(self, arguments=None, continue_on=None, dependencies=None, depends=None, name=None, on_exit=None, template=None, template_ref=None, when=None, with_items=None, with_param=None, with_sequence=None, hooks=None, local_vars_configuration=None):  # noqa: E501
        """V1alpha1DAGTask - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._arguments = None
        self._continue_on = None
        self._dependencies = None
        self._depends = None
        self._name = None
        self._on_exit = None
        self._template = None
        self._template_ref = None
        self._when = None
        self._with_items = None
        self._with_param = None
        self._with_sequence = None
        self._hooks = None
        self.discriminator = None

        if arguments is not None:
            self.arguments = arguments
        if continue_on is not None:
            self.continue_on = continue_on
        if dependencies is not None:
            self.dependencies = dependencies
        if depends is not None:
            self.depends = depends
        self.name = name
        if on_exit is not None:
            self.on_exit = on_exit
        if template is not None:
            self.template = template
        if template_ref is not None:
            self.template_ref = template_ref
        if when is not None:
            self.when = when
        if with_items is not None:
            self.with_items = with_items
        if with_param is not None:
            self.with_param = with_param
        if with_sequence is not None:
            self.with_sequence = with_sequence
        if hooks is not None:
            self.hooks = hooks

    @property
    def arguments(self):
        """Gets the arguments of this V1alpha1DAGTask.  # noqa: E501


        :return: The arguments of this V1alpha1DAGTask.  # noqa: E501
        :rtype: V1alpha1Arguments
        """
        return self._arguments

    @arguments.setter
    def arguments(self, arguments):
        """Sets the arguments of this V1alpha1DAGTask.


        :param arguments: The arguments of this V1alpha1DAGTask.  # noqa: E501
        :type: V1alpha1Arguments
        """

        self._arguments = arguments

    @property
    def continue_on(self):
        """Gets the continue_on of this V1alpha1DAGTask.  # noqa: E501


        :return: The continue_on of this V1alpha1DAGTask.  # noqa: E501
        :rtype: V1alpha1ContinueOn
        """
        return self._continue_on

    @continue_on.setter
    def continue_on(self, continue_on):
        """Sets the continue_on of this V1alpha1DAGTask.


        :param continue_on: The continue_on of this V1alpha1DAGTask.  # noqa: E501
        :type: V1alpha1ContinueOn
        """

        self._continue_on = continue_on

    @property
    def dependencies(self):
        """Gets the dependencies of this V1alpha1DAGTask.  # noqa: E501

        Dependencies are name of other targets which this depends on  # noqa: E501

        :return: The dependencies of this V1alpha1DAGTask.  # noqa: E501
        :rtype: list[str]
        """
        return self._dependencies

    @dependencies.setter
    def dependencies(self, dependencies):
        """Sets the dependencies of this V1alpha1DAGTask.

        Dependencies are name of other targets which this depends on  # noqa: E501

        :param dependencies: The dependencies of this V1alpha1DAGTask.  # noqa: E501
        :type: list[str]
        """

        self._dependencies = dependencies

    @property
    def depends(self):
        """Gets the depends of this V1alpha1DAGTask.  # noqa: E501

        Depends are name of other targets which this depends on  # noqa: E501

        :return: The depends of this V1alpha1DAGTask.  # noqa: E501
        :rtype: str
        """
        return self._depends

    @depends.setter
    def depends(self, depends):
        """Sets the depends of this V1alpha1DAGTask.

        Depends are name of other targets which this depends on  # noqa: E501

        :param depends: The depends of this V1alpha1DAGTask.  # noqa: E501
        :type: str
        """

        self._depends = depends

    @property
    def name(self):
        """Gets the name of this V1alpha1DAGTask.  # noqa: E501

        Name is the name of the target  # noqa: E501

        :return: The name of this V1alpha1DAGTask.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this V1alpha1DAGTask.

        Name is the name of the target  # noqa: E501

        :param name: The name of this V1alpha1DAGTask.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def on_exit(self):
        """Gets the on_exit of this V1alpha1DAGTask.  # noqa: E501

        OnExit is a template reference which is invoked at the end of the template, irrespective of the success, failure, or error of the primary template.  # noqa: E501

        :return: The on_exit of this V1alpha1DAGTask.  # noqa: E501
        :rtype: str
        """
        return self._on_exit

    @on_exit.setter
    def on_exit(self, on_exit):
        """Sets the on_exit of this V1alpha1DAGTask.

        OnExit is a template reference which is invoked at the end of the template, irrespective of the success, failure, or error of the primary template.  # noqa: E501

        :param on_exit: The on_exit of this V1alpha1DAGTask.  # noqa: E501
        :type: str
        """

        self._on_exit = on_exit

    @property
    def template(self):
        """Gets the template of this V1alpha1DAGTask.  # noqa: E501

        Name of template to execute  # noqa: E501

        :return: The template of this V1alpha1DAGTask.  # noqa: E501
        :rtype: str
        """
        return self._template

    @template.setter
    def template(self, template):
        """Sets the template of this V1alpha1DAGTask.

        Name of template to execute  # noqa: E501

        :param template: The template of this V1alpha1DAGTask.  # noqa: E501
        :type: str
        """

        self._template = template

    @property
    def template_ref(self):
        """Gets the template_ref of this V1alpha1DAGTask.  # noqa: E501


        :return: The template_ref of this V1alpha1DAGTask.  # noqa: E501
        :rtype: V1alpha1TemplateRef
        """
        return self._template_ref

    @template_ref.setter
    def template_ref(self, template_ref):
        """Sets the template_ref of this V1alpha1DAGTask.


        :param template_ref: The template_ref of this V1alpha1DAGTask.  # noqa: E501
        :type: V1alpha1TemplateRef
        """

        self._template_ref = template_ref

    @property
    def when(self):
        """Gets the when of this V1alpha1DAGTask.  # noqa: E501

        When is an expression in which the task should conditionally execute  # noqa: E501

        :return: The when of this V1alpha1DAGTask.  # noqa: E501
        :rtype: str
        """
        return self._when

    @when.setter
    def when(self, when):
        """Sets the when of this V1alpha1DAGTask.

        When is an expression in which the task should conditionally execute  # noqa: E501

        :param when: The when of this V1alpha1DAGTask.  # noqa: E501
        :type: str
        """

        self._when = when

    @property
    def with_items(self):
        """Gets the with_items of this V1alpha1DAGTask.  # noqa: E501

        WithItems expands a task into multiple parallel tasks from the items in the list  # noqa: E501

        :return: The with_items of this V1alpha1DAGTask.  # noqa: E501
        :rtype: list[object]
        """
        return self._with_items

    @with_items.setter
    def with_items(self, with_items):
        """Sets the with_items of this V1alpha1DAGTask.

        WithItems expands a task into multiple parallel tasks from the items in the list  # noqa: E501

        :param with_items: The with_items of this V1alpha1DAGTask.  # noqa: E501
        :type: list[object]
        """

        self._with_items = with_items

    @property
    def with_param(self):
        """Gets the with_param of this V1alpha1DAGTask.  # noqa: E501

        WithParam expands a task into multiple parallel tasks from the value in the parameter, which is expected to be a JSON list.  # noqa: E501

        :return: The with_param of this V1alpha1DAGTask.  # noqa: E501
        :rtype: str
        """
        return self._with_param

    @with_param.setter
    def with_param(self, with_param):
        """Sets the with_param of this V1alpha1DAGTask.

        WithParam expands a task into multiple parallel tasks from the value in the parameter, which is expected to be a JSON list.  # noqa: E501

        :param with_param: The with_param of this V1alpha1DAGTask.  # noqa: E501
        :type: str
        """

        self._with_param = with_param

    @property
    def with_sequence(self):
        """Gets the with_sequence of this V1alpha1DAGTask.  # noqa: E501


        :return: The with_sequence of this V1alpha1DAGTask.  # noqa: E501
        :rtype: V1alpha1Sequence
        """
        return self._with_sequence

    @with_sequence.setter
    def with_sequence(self, with_sequence):
        """Sets the with_sequence of this V1alpha1DAGTask.


        :param with_sequence: The with_sequence of this V1alpha1DAGTask.  # noqa: E501
        :type: V1alpha1Sequence
        """

        self._with_sequence = with_sequence

    @property
    def hooks(self):
        """Gets the hooks of this V1alpha1DAGTask.  # noqa: E501


        :return: The hooks of this V1alpha1DAGTask.  # noqa: E501
        :rtype: V1alpha1Sequence
        """
        return self._hooks

    @hooks.setter
    def hooks(self, hooks):
        """Sets the hooks of this V1alpha1DAGTask.


        :param hooks: The hooks of this V1alpha1DAGTask.  # noqa: E501
        :type: V1alpha1Sequence
        """

        self._hooks = hooks

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1alpha1DAGTask):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1alpha1DAGTask):
            return True

        return self.to_dict() != other.to_dict()
